# -*- coding: utf-8 -*-
from django import forms
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import ugettext

from registration.forms import RegistrationFormUniqueEmail

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Div, Submit
from crispy_forms.bootstrap import FormActions

from captcha.fields import CaptchaField

from sveeaccounts.models import UserProfileBase

class RegistrationWithCaptchaForm(RegistrationFormUniqueEmail):
    captcha = CaptchaField()
    
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.form_style = 'inline'
        self.helper.add_input(Submit('submit', ugettext('Continue')))
        
        super(RegistrationWithCaptchaForm, self).__init__(*args, **kwargs)

class LoginForm(AuthenticationForm):
    
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.form_style = 'inline'
        self.helper.form_class = 'tiny'
        self.helper.add_input(Submit('submit', ugettext('Ok')))
        
        super(LoginForm, self).__init__(*args, **kwargs)

def UserProfileBaseLayout():
    """
    Return the default layout, this must be wrapped in a function to correct 
    translations
    """
    return Layout(
        Fieldset(
            ugettext('account'),
            'new_password1',
            'new_password2',
        ),
        Fieldset(
            ugettext('identity'),
            'first_name',
            'last_name',
            'email',
            'adress',
            'town',
            'zipcode',
            'phone_number',
            'mobile_number',
        ),
    )
    
class UserProfileBaseForm(forms.ModelForm):
    """
    User profile form
    """
    # Bind only some fields from the user model
    first_name = forms.CharField(label=ugettext('first name'), max_length=30, required=True)
    last_name = forms.CharField(label=ugettext("last name"), max_length=30, required=True)
    email = forms.EmailField(label=ugettext("E-mail"), max_length=75, required=True)
    new_password1 = forms.CharField(label=ugettext("Password"), widget=forms.PasswordInput, required=False)
    new_password2 = forms.CharField(label=ugettext("Password (again)"), widget=forms.PasswordInput, required=False)
    
    def __init__(self, *args, **kwargs):
        self.author = kwargs.pop('user')
        
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.form_class = 'form-horizontal well'
        self.helper.form_style = 'inline'
        self.helper.layout = Layout(
            kwargs.pop('layout', UserProfileBaseLayout()),
            FormActions(
                Submit('submit', 'Submit'),
            )
        )
        
        super(UserProfileBaseForm, self).__init__(*args, **kwargs)

    def clean_new_password2(self):
        password1 = self.cleaned_data.get('new_password1')
        password2 = self.cleaned_data.get('new_password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(ugettext("The two password fields didn't match."))
        return password2
    
    def save(self, *args, **kwargs):
        instance = super(UserProfileBaseForm, self).save(commit=True, *args, **kwargs)
        
        u = instance.user
        u.first_name = self.cleaned_data["first_name"]
        u.last_name = self.cleaned_data["last_name"]
        u.email = self.cleaned_data["email"]
        u.save()
        if self.cleaned_data.get("new_password2", None) and len(self.cleaned_data["new_password2"].strip())>0:
            u.set_password(self.cleaned_data["new_password2"])
        
        return instance
    
    class Meta:
        model = UserProfileBase
        exclude = ('user',)



# -*- coding: utf-8 -*-
"""
Application Crumbs
"""
from autobreadcrumbs import site
from django.utils.translation import ugettext_lazy

site.update({
    'registration_activation_complete': ugettext_lazy('Registration activation success'),
    'registration_activate': ugettext_lazy('Registration activation fail'),
    'registration_register': ugettext_lazy('Register'),
    'registration_complete': ugettext_lazy('Registration complete'),
    'registration_disallowed': ugettext_lazy('Registration disallowed'),
    'auth_login': ugettext_lazy('Login'),
})



"""
URLconf for WithCaptchaBackend behaviors
"""
from django.conf import settings
from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template
from django.contrib.auth import views as auth_views

from registration.views import activate
from registration.views import register

from sveeaccounts.forms import LoginForm

REGISTRATION_BLOCKED = getattr(settings, 'REGISTRATION_BLOCKED', False)

if REGISTRATION_BLOCKED:
    urlpatterns = patterns('',
        url(r'^activate/complete/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_activation_complete'),
        url(r'^activate/(?P<activation_key>\w+)/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_activate'),
        url(r'^register/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_register'),
        url(r'^register/complete/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_complete'),
        url(r'^register/closed/$', direct_to_template, {'template': 'registration/blocked.html'}, name='registration_disallowed'),

        url(r'^login/$', auth_views.login, {'template_name': 'registration/login.html', 'authentication_form': LoginForm}, name='auth_login'),
        url(r'^logout/$', auth_views.logout_then_login, {'login_url': '/'}, name='auth_logout'),
    )
else:
    urlpatterns = patterns('',
        url(r'^activate/complete/$',
            direct_to_template,
            {'template': 'registration/activation_complete.html'},
            name='registration_activation_complete'),
        # Activation keys get matched by \w+ instead of the more specific
        # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
        # that way it can return a sensible "invalid key" message instead of a
        # confusing 404.
        url(r'^activate/(?P<activation_key>\w+)/$',
            activate,
            {'backend': 'sveeaccounts.backends.WithCaptchaBackend'},
            name='registration_activate'),
        url(r'^register/$',
            register,
            {'backend': 'sveeaccounts.backends.WithCaptchaBackend'},
            name='registration_register'),
        url(r'^register/complete/$',
            direct_to_template,
            {'template': 'registration/registration_complete.html'},
            name='registration_complete'),
        url(r'^register/closed/$',
            direct_to_template,
            {'template': 'registration/registration_closed.html'},
            name='registration_disallowed'),

        url(r'^login/$',
            auth_views.login,
            {'template_name': 'registration/login.html', 'authentication_form': LoginForm},
            name='auth_login'),
        url(r'^logout/$',
            auth_views.logout_then_login,
            {'login_url': '/'},
            name='auth_logout'),
    )



# -*- coding: utf-8 -*-
from registration.backends.default import DefaultBackend

from sveeaccounts.forms import RegistrationWithCaptchaForm

class WithCaptchaBackend(DefaultBackend):
    def get_form_class(self, request):
        """
        Return the default form class used for user registration.
        
        """
        return RegistrationWithCaptchaForm



# -*- coding: utf-8 -*-
"""
Accounts views
"""
from django.views.generic import UpdateView
from django.core.urlresolvers import reverse

from braces.views import UserFormKwargsMixin, LoginRequiredMixin

from sveeaccounts.models import UserProfileBase
from sveeaccounts.forms import UserProfileBaseForm

class MyAccountBaseView(LoginRequiredMixin, UserFormKwargsMixin, UpdateView):
    """
    The form view to edit the user account and his profile
    
    The following attributes are required and you have also to define a 
    'get_success_url' method.
    """
    template_name = None # Required
    form_class = UserProfileBaseForm
    model = None # Required. This should be assigned with your profile model
    
    def get_object(self, *args, **kwargs):
        return self.request.user.get_profile()
    
    def get_initial(self):
        """
        Returns the initial data to use for forms on this view.
        """
        return {
            'first_name': self.request.user.first_name,
            'last_name': self.request.user.last_name,
            'email': self.request.user.email,
        }



# -*- coding: utf-8 -*-
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
import types

class UserProfileBase(models.Model):
    u"""
    User Profile to extend the User model
    """
    user = models.ForeignKey(User, unique=True, blank=False)
    adress = models.TextField(_('adress'), blank=False)
    town = models.CharField(_('town'), max_length=75, blank=False)
    zipcode = models.CharField(_('zipcode'), max_length=6, blank=False)
    phone_number = models.CharField(_('phone'), max_length=20, blank=False)
    mobile_number = models.CharField(_('mobile'), max_length=20, blank=True, null=True)
    
    def __unicode__(self):
        return u"{0}'s profile".format(self.user.username)
    
    class Meta:
        abstract = True
        verbose_name = _("user profile")
        verbose_name_plural = _("user profiles")

def append_profile_on_user_create(sender, instance, created, **kwargs):
    """
    Automatically add an empty profile on user create if it doesn't exist yet
    
    Need to be connected to model with something like that : ::
    
        from django.contrib.auth.models import User
        from django.db.models.signals import post_save as post_save_signal
        post_save_signal.connect(append_profile_on_user_create, sender=User)
        
    "extra_fields" can be passed in kwargs, it will contain extra fields (like these 
    appended by inherit the abstract model) with their default value. Also theses values 
    can be a lambda which receive the user instance and return a correct value.
    """
    if created:
        try:
            instance.get_profile()
        except ObjectDoesNotExist:
            extra_fields = kwargs.get('extra_fields', {})
            
            for k,v in extra_fields.items():
                if isinstance(v, types.FunctionType):
                    extra_fields[k] = v(instance)
            
            instance.userprofile_set.create(
                adress='placeholder',
                town='placeholder',
                zipcode='empty',
                phone_number='placeholder',
                **extra_fields
            )
